/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (c) 2004 - 2005, Tilghman Lesher.  All rights reserved.
 *
 * Philipp Dunkel <philipp.dunkel@ebox.at>
 *
 * This code is released by the author with no restrictions on usage.
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 */

/*! \file
 *
 * \brief TryExec application
 *
 * \author Philipp Dunkel <philipp.dunkel@ebox.at>
 *
 * \ingroup applications
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")

#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/options.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"

/* Maximum length of any variable */
#define MAXRESULT	1024

static char *tdesc = "Executes applications and always return 0";

static char *app_tryexec = "TryExec";

static char *tryexec_synopsis = "Executes internal application and always returns 0";

static char *tryexec_descrip =
"Usage: TryExec(appname(arguments))\n"
"  Allows an arbitrary application to be invoked even when not\n"
"hardcoded into the dialplan. To invoke external applications\n"
"see the application System. Alway returns 0.\n"
"The channel variable TRYSTATUS is set to:\n"
"\t SUCCESS if the application returned 0\n"
"\t FAILED if the application returned != 0\n"
"\t NOAPP if the application was not found or was not specified\n"
"\t NOMEMORY if there was not enough memory to execute.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int tryexec_exec(struct ast_channel *chan, void *data)
{
	 int res=0;
        struct localuser *u;
        char *s, *appname, *endargs, args[MAXRESULT];
        struct ast_app *app;

        LOCAL_USER_ADD(u);

        memset(args, 0, MAXRESULT);

        /* Check and parse arguments */
        if (data) {
                s = ast_strdupa((char *)data);
                if (s) {
                        appname = strsep(&s, "(");
                        if (s) {
                                endargs = strrchr(s, ')');
                                if (endargs)
                                        *endargs = '\0';
                                pbx_substitute_variables_helper(chan, s, args, MAXRESULT - 1);
                        }
                        if (appname) {
                                app = pbx_findapp(appname);
                                if (app) {
                                        res = pbx_exec(chan, app, args, 1);
                                        pbx_builtin_setvar_helper(chan, "TRYSTATUS", res?"FAILED":"SUCCESS");
                                } else {
                                        ast_log(LOG_WARNING, "Could not find application (%s)\n", appname);
                                        pbx_builtin_setvar_helper(chan, "TRYSTATUS", "NOAPP");
                                }
                        } else {
                                ast_log(LOG_WARNING, "No application specified!\n");
                                pbx_builtin_setvar_helper(chan, "TRYSTATUS", "NOAPP");
                        }
                } else {
                        ast_log(LOG_ERROR, "Out of memory\n");
                        pbx_builtin_setvar_helper(chan, "TRYSTATUS", "NOMEMORY");
                }
        }

        LOCAL_USER_REMOVE(u);
        return 0;
}

int unload_module(void)
{
	int res;

	res = ast_unregister_application(app_tryexec);

	STANDARD_HANGUP_LOCALUSERS;

	return res;
}

int load_module(void)
{
	return ast_register_application(app_tryexec, tryexec_exec, tryexec_synopsis, tryexec_descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
